#!/usr/bin/env sh

#
# Copyright(c) 2006-2021 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE: All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and 
# technical concepts contained herein are proprietary to ThreatSTOP, Inc. 
# and its suppliers and certain aspects thereof are protected by United 
# States Patent No. 8,533,822 and United States Patent No. 8,869,237, 
# and are protected by trade secret or copyright law.
#
# US Government Entities: The ThreatSTOP service, software and
# documentation, as applicable, are "commercial computer software" and
# "commercial computer software documentation" developed exclusively at
# private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
# 12.212 or DFARS 227 7202 and their successors, as applicable, use,
# reproduction and disclosure of the software and documentation is
# governed by the terms of Threatstop's commercial agreement.
#

#
# ThreatSTOP OpenBSD PF Log Uploading Script
#
# Send PF log file to ThreatSTOP. The log file is converted to text
# and uploaded to ThreatSTOP using the loguploadclient.pl Perl script.
# File is POSTed to the ThreatSTOP HTTPS website.
#
# This script must be run as root due to premissions placed on the pflog
# files.
#
# Script works with logs that are compressed and not compressed. The PFLOG 
# setting in the threatstop.conf file is the pflog that will be converted 
# to text. By default, the configuration is set to look for the last rotated 
# file, named /var/log/pflog.0 by default.
#
###############################################################################


PREFIX="/usr/local"
CONFIG="$PREFIX/etc/threatstop.conf"
LAST_LOG="$PREFIX/etc/last_log_sent"
WORKDIR="/tmp"
SEND_HISTORY="/var/log/threatstop_sendlog_history.log"


echoerr() { printf "%s\n" "$*" >&2; }

# Check for and read the config file, abort if not found
if [ ! -f $CONFIG ]
then
	echoerr "ALERT: Cannot read config file: \"$CONFIG\" Exiting"
	exit 1
else
	. $CONFIG
fi

if [ -z $device_id ];then
	echoerr "Error: No 'device_id' was found in '$CONFIG', can't continue..."
	exit 2
fi

# Check to make sure script is run by root
if [ `id -u` != "0" ]
then
	echoerr "You must run the script as root. Exiting"
	exit 1
fi

# We need to keep track of the last log sent so we don't resend the same log over and over
log_new_or_exit() {
	LOG=$1
	if [ -f $LAST_LOG ];then
		LAST_LOG_MD5=$(cat $LAST_LOG|tr -d ' ')
		echo "Loaded last log MD5 ($LAST_LOG_MD5)"
	fi
	LOG_MD5=$(md5 $LOG| cut -d'=' -f2|tr -d ' ')
	echo "Read current log MD5 ($LOG_MD5)"
	if [ "[$LOG_MD5]" = "[$LAST_LOG_MD5]" ];then
		echo "Log has already been processed.... [skipping]"
		exit 4
	else
		echo "Processing new log file @ `date`..." | tee -a $SEND_HISTORY
		echo "$LOG_MD5" > $LAST_LOG
	fi

}


# Check to see if the file is compressed.
if [ -f $PFLOG ]
then
	log_new_or_exit $PFLOG

	# not compressed
	cp $PFLOG $WORKDIR
elif [ -f $PFLOG.gz ]
then
	log_new_or_exit $PFLOG.gz
	# compressed
	cp $PFLOG.gz $WORKDIR
	GZLOG=`basename $WORKDIR/pflog.0.gz`
	gunzip $WORKDIR/$GZLOG
else
	echo "logfile ($PFLOG) not found... "
	exit 1
fi

LOG="$WORKDIR/pflog.0"

# use tcpdump to dump log to a file
if uname -s | grep -q -i 'freebsd';then
	# We add a 't' for FreeBSD to output full dates
	tcpdump -nettttr $LOG 2>/dev/null > $logfile
else
	# Default OpenBSD format
    tcpdump -n -e -ttt -r $LOG 2>/dev/null > $logfile
fi

# Run log upload Perl script
if perl $PREFIX/sbin/loguploadclient.pl >> $SEND_HISTORY 2>&1;then
	echo "Send $PFLOG [Success]" | tee -a $SEND_HISTORY
else
	echoerr "Send $PFLOG [Fail]" | tee -a $SEND_HISTORY
fi


# delete work files
rm $WORKDIR/pflog.0 $logfile

